package cz.moneta.test.regression.backoffice.exevido.web.mailroom;

import cz.moneta.test.dsl.Harness;
import cz.moneta.test.dsl.util.credentials.Keys;
import cz.moneta.test.harness.annotations.JiraTestCase;
import cz.moneta.test.harness.annotations.TestCase;
import cz.moneta.test.harness.annotations.TestScenario;
import cz.moneta.test.harness.support.auth.Credentials;
import cz.moneta.test.harness.support.auth.Key;
import org.junit.jupiter.api.AfterEach;

import static cz.moneta.test.dsl.exevido.pages.NewOutgoingMessagePage.ATTACHMENT_PATH;
import static cz.moneta.test.dsl.exevido.pages.NewOutgoingMessagePage.OUTGOING_MESSAGE_ID_KEY;
import static cz.moneta.test.dsl.util.task.ufo.cb.CommercialTasks.JIRA_COMMENT_STORE_KEY;

@TestScenario(name = "Send outgoing message to multiple recipients")
public class SendOutgoingMessageToMultipleRecipients {

    @TestCase(name = "Send outgoing message to multiple recipients")
    @JiraTestCase(id = "BOC-T8519")
    public void sendOutgoingMessageToMultipleRecipients(Harness harness, @Key(Keys.exevido.mailroom) Credentials credentials) {
        harness.withExevido()
                .openLoginPage()
                .then(harness.tasks().exevidoTasks().login().withCredentialsToMmb(credentials))
                .clickOutgoingMessages()
                .clickAllOutgoingMessages()
                .clickCreateNewOutgoingMessage()
                .uploadAttachment(ATTACHMENT_PATH)
                .clickMultipleRecipients()
                .fillMultipleRecipient("42uf8wu")
                .pressEnter()
                .addRecipient()
                .fillMultipleRecipient("uchhtu9")
                .pressEnter()
                .addRecipient()
                .fillSubject("Automat TEST")
                .fillAssigneeUserGroup("F00102")
                .clickSendMessageDropdown()
                .clickSendMessageNow()
                .checkMassMessageCorrectlySend()
                .storeMessageId();
    }

    @AfterEach
    protected void tearDown(Harness harness) {
        harness.store(JIRA_COMMENT_STORE_KEY,
                "\nMessage ID: " + harness.get(OUTGOING_MESSAGE_ID_KEY, true));
    }
}